def translate_chatgpt(
    model_name: str = "gpt-3.5-turbo",
    input_language: str = "English",
    output_language: str = "Turkish",
    text: str = "Hello, how are you?",
    temperature: float = 0.0,
):
    """
    This function is a template for a chatbot that translates between two languages.
    Args:
        model_name: The name of the model to use. Defaults to "gpt-3.5-turbo".
        input_language: The language to translate from. Defaults to "English".
        output_language: The language to translate to. Defaults to "Turkish".
        text: The text to translate. Defaults to "Hello, how are you?".
        temperature: The temperature to use for the model. Defaults to 0.0.
    Returns:
        The translated text.
    """
    from langchain.chat_models import ChatOpenAI
    from langchain.prompts.chat import (
        ChatPromptTemplate,
        HumanMessagePromptTemplate,
        SystemMessagePromptTemplate,
    )

    chat = ChatOpenAI(model_name=model_name, temperature=temperature)
    template = "You are a helpful assistant that translates {input_language} to {output_language}."
    system_message_prompt = SystemMessagePromptTemplate.from_template(template)
    human_template = "{text}"
    human_message_prompt = HumanMessagePromptTemplate.from_template(
        human_template
    )

    chat_prompt = ChatPromptTemplate.from_messages(
        [system_message_prompt, human_message_prompt]
    )

    chat_completion = chat(
        chat_prompt.format_prompt(
            input_language=input_language,
            output_language=output_language,
            text=text,
        ).to_messages()
    )
    last_ai_message = chat_completion.content

    return last_ai_message



def custom_prompt_template(
    model_name: str = "gpt-3.5-turbo",
    template: str = "You are a helpful assistant that English to Turkish and you are asked to translate the following text: {text}",
    input_variables: str = "text",
    text: str = "Hello, how are you?",
    temperature: float = 0.0,
):
    from langchain.chains import LLMChain
    from langchain.chat_models import ChatOpenAI
    from langchain.prompts import PromptTemplate

    llm = ChatOpenAI(model_name=model_name, temperature=temperature)

    prompt = PromptTemplate(
        input_variables=[input_variables], template=template
    )
    chain = LLMChain(llm=llm, prompt=prompt)
    output = chain.run(text)
    return output



def promptlayer_chatgpt(
    text: str = "Hello, I am a chatbot and",
    model_name: str = "gpt-3.5-turbo",
    temperature: float = 0.0,
):
    """
    This function is a template for a chatbot that uses PromptLayer.
    Args:
        text: The text to use as the prompt. Defaults to "Hello, I am a chatbot and".
        model_name: The name of the model to use. Defaults to "gpt-3.5-turbo".
        temperature: The temperature to use for the model. Defaults to 0.0.
    Returns:
        The chatbot's response.
    """

    from langchain.chat_models import PromptLayerChatOpenAI
    from langchain.schema import HumanMessage

    chat = PromptLayerChatOpenAI(
        pl_tags=["langchain"], model_name=model_name, temperature=temperature
    )
    chat_completion = chat([HumanMessage(content=text)]).content

    return chat_completion



def load_openai_key(openai_key):
    import os

    os.environ["OPENAI_API_KEY"] = openai_key


def load_promptlayer_key(promptlayer_key):
    import os

    os.environ["PROMPTLAYER_API_KEY"] = promptlayer_key



from chatgpthub.generate import ChatGptHubDemo
from chatgpthub.templates import custom_prompt_template, translate_chatgpt
from chatgpthub.tools import load_csv_agent, promptlayer_chatgpt
from chatgpthub.utils import load_openai_key, load_promptlayer_key

__version__ = "0.1.2"



def load_csv_agent(csv_file, text, model_name="gpt-3.5-turbo", temperature=0.0):
    from langchain.agents import create_csv_agent
    from langchain.chat_models import ChatOpenAI

    model = ChatOpenAI(
        model_name=model_name,
        temperature=temperature,
    )
    langchain_agent = create_csv_agent(
        llm=model,
        path=csv_file,
    )
    langchain_agent.agent.llm_chain.prompt.template

    return langchain_agent.run(text)



from typing import Optional

from chatgpthub import (
    load_openai_key,
    load_promptlayer_key,
    prompt_template,
    promptlayer_chatgpt,
    translate_chatgpt,
)


def load_key(
    openai_key: Optional[str] = None, promptlayer_key: Optional[str] = None
):
    """
    This function loads the OpenAI and PromptLayer keys.
    Args:
        openai_key: The OpenAI key to use. Defaults to None.
        promptlayer_key: The PromptLayer key to use. Defaults to None.
    """
    if openai_key is not None:
        load_openai_key(openai_key)
    if promptlayer_key is not None:
        load_promptlayer_key(promptlayer_key)


class ChatGptHubDemo:
    def __init__(
        self,
        openai_key: Optional[str] = None,
        promptlayer_key: Optional[str] = None,
    ):
        """
        This class is a demo for the chatgpthub library.
        Args:
            openai_key: The OpenAI key to use. Defaults to None.
            promptlayer_key: The PromptLayer key to use. Defaults to None.
        """
        load_key(openai_key=openai_key)

        self.promptlayer_key = promptlayer_key

    def translate(
        self,
        model_name: str = "gpt-3.5-turbo",
        input_language: str = "English",
        output_language: str = "Turkish",
        text: str = "Hello, how are you?",
        temperature: float = 0.0,
    ):
        """
        This function is a demo for the translate_chatgpt function.
        Args:
            model_name: The name of the model to use. Defaults to "gpt-3.5-turbo".
            input_language: The language to translate from. Defaults to "English".
            output_language: The language to translate to. Defaults to "Turkish".
            text: The text to translate. Defaults to "Hello, how are you?".
            temperature: The temperature to use for the model. Defaults to 0.0.
        Returns:
            The translated text.
        """
        output = translate_chatgpt(
            model_name=model_name,
            input_language=input_language,
            output_language=output_language,
            text=text,
            temperature=temperature,
        )

        return output

    def custom_template(
        self,
        model_name: str = "gpt-3.5-turbo",
        template: str = "You are a helpful assistant that English to Turkish and you are asked to translate the following text: {text}",
        input_variables: str = "text",
        text: str = "Hello, how are you?",
        temperature: float = 0.0,
    ):
        """
        This function is a demo for the prompt_template function.
        Args:
            model_name: The name of the model to use. Defaults to "gpt-3.5-turbo".
            template: The prompt template to use. Defaults to "You are a helpful assistant that English to Turkish and you are asked to translate the following text: {text}".
            input_variables: The input variables to use. Defaults to "text".
            text: The text to translate. Defaults to "Hello, how are you?".
            temperature: The temperature to use for the model. Defaults to 0.0.
        Returns:
            The translated text.
        """
        output = prompt_template(
            model_name=model_name,
            template=template,
            input_variables=input_variables,
            text=text,
            temperature=temperature,
        )

        return output

    def promptlayer(
        self,
        model_name: str = "gpt-3.5-turbo",
        text: str = "Hello, I am a chatbot and",
        temperature: float = 0.0,
    ):
        """
        This function is a demo for the promptlayer_chatgpt function.
        Args:
            model_name: The name of the model to use. Defaults to "gpt-3.5-turbo".
            text: The text to use as the prompt. Defaults to "Hello, I am a chatbot and".
            temperature: The temperature to use for the model. Defaults to 0.0.
        Returns:
            The chatbot's response.
        """
        if self.promptlayer_key is not None:
            load_promptlayer_key(self.promptlayer_key)
        else:
            raise Exception(
                "You need to provide a PromptLayer key to use the promptlayer_chatgpt function."
            )

        output = promptlayer_chatgpt(
            model_name=model_name, text=text, temperature=temperature
        )

        return output
